{{
*****************************************************
* Debug LED17 Decimal driver v1.2 (1wire version)   *
* Shows one one hub address as decimal              *
* Author: Tony Philipsson                           *
* Copyright 2014 Lime TechMill LLC                  *
*****************************************************
}}
CON
    Pin= 1<<0              'P0 the single clk/data pin, for P8 use 1<<8 etc
    A1=  %1<<1             'the 17 LED Segments
    A2=  %1<<2
    B=   %1<<3
    C=   %1<<5
    D1=  %1<<6
    D2=  %1<<7
    E=   %1<<9
    F=   %1<<10
    G1=  %1<<21
    G2=  %1<<15
    H=   %1<<11
    I=   %1<<13
    J=   %1<<14
    K=   %1<<17
    L=   %1<<18
    M=   %1<<19
    DP=  %1<<22

PUB Display (hubaddress)           'get the hub address to be passed on to PAR
    cognew(@asm_entry, hubaddress) 'launch assembly program in a COG    

DAT
              org  0

asm_entry     mov       dira,clkpin                     'make only clk pin an output
              mov       outa,clkpin                     'set all pins in this cog to low but CLKpin high               
              mov       digitdelay,delay2ms              
              add       digitdelay,cnt                  'copy cnt digitdelay

main          mov       digit,#6                        'we have 6 digits to multiplex
              mov       sink, #1                        'reset mosfet-to-sink pointer
              movs      decimal,#table                  'start at 100'000
              rdlong    myvalue,PAR                     'get var from hub address  
              cmp       maxvalue,myvalue wc             'over max value?
        if_c  mov       myvalue,#0                      'clear it so decimal loop don't take to long
              muxc      error,#1                        'set or clear bit0 as error overflow flag
              
loop1         waitcnt   digitdelay,delay2ms             '2 ms  
              movs      fontpnt,#font                   'start at "0"       
decimal       cmpsub    myvalue,0-0 wc                  'try 100'000 first
        if_c  add       fontpnt,#1
        if_c  jmp       #decimal            
              mov       bit_test,bit23                  'reset bit mask so bit 23 is only set
fontpnt       mov       serial,0-0                      'start a new serial data
              test      error,#1 wz                     'error overflow set?
        if_nz mov       serial,fontE                    'replace with letter E          
              or        serial,sink                     'fuse in the transistor sink bit
                                                        'inner loop starts here, send 23 clk pulses plus a long latch
loop2         test      serial,bit_test wz              'test serial data bitwize, set z                                
        if_nz mov       cnt,delay1                      'a quick clk, move 1uS to Shadow Cnt.
        if_z  mov       cnt,delay15                     'a long clk (15uS), let SER dip low
              add       cnt,cnt                         'add current cnt to shadow cnt. 
              andn      outa,clkpin                     'set pin low 
              waitcnt   cnt,delay30                     'wait 1uS or 15uS , add 30us to cnt when done.
              or        outa,clkpin                     'set pin high   
              waitcnt   cnt,#0                          'wait 30uS , add 0 us to cnt when done.               
              shr       bit_test,#1 wz                  'shift right
        if_nz jmp       #loop2                          'if not zero, jmp 
         
              mov       cnt,delay200                    'prepare a 200us delay to letting Latch go low. 
              add       cnt, cnt                        'add cnt to shadow cnt
              andn      outa,clkpin                     'set pin low
              waitcnt   cnt,#0 
              or        outa,clkpin                     'set pin high
              shl       sink,#4                         'next digit
              add       decimal,#1                      'try 10'000 next    
              djnz      digit, #loop1                   'have we done 6 digits?
              jmp       #main                           'restart

font          long      A1+A2+B+C+D1+D2+E+F+J+M         '0
              long      B+C                             '1
              long      A1+A2+B+D1+D2+E+G1+G2           '2
              long      A1+A2+B+C+D1+D2+G2              '3
              long      B+C+F+G1+G2                     '4
              long      A1+A2+D1+D2+F+G1+K              '5
              long      A1+A2+C+D1+D2+E+F+G1+G2         '6
              long      A1+A2+B+C                       '7
              long      A1+A2+B+C+D1+D2+E+F+G1+G2       '8
              long      A1+A2+B+C+D1+D2+F+G1+G2         '9
fontE         long      A1+A2+D1+D2+E+F+G1+G2           'E 
bit23         long      |<23                            'same as %1<<23,  serial bits to shift out
delay1        long      80                              '1us  (80 MHz) 
delay15       long      80*15                           '15us (80 MHz)
delay30       long      80*30                           '30us (80 MHz) 
delay200      long      80*200                          '200us (80 MHz)
delay2ms      long      80*1000*2                       '2ms (80 Mhz)
clkpin        long      Pin                             'value from constant above
maxvalue      long      999999
table         long      100_000, 10_000, 1_000, 100, 10, 1
digitdelay    res       1  
sink          res       1                               'what transistor to sink
myvalue       res       1
error         res       1
bit_test      res       1
serial        res       1
digit         res       1